import { defineNuxtModule, resolvePath, addPluginTemplate } from '@nuxt/kit';

const fileExt = /\.s[a|c]ss$/;
function getScssTransformPlugin(sassVariables) {
  const sassImportCode = ["@import 'quasar/src/css/variables.sass'", ""];
  if (typeof sassVariables === "string") {
    sassImportCode.unshift(`@import '${sassVariables}'`);
  }
  return {
    name: "vite:quasar:scss",
    enforce: "pre",
    transform(src, id) {
      const shouldTransform = id.match(fileExt)?.[0] || false;
      if (shouldTransform) {
        const prefix = shouldTransform === ".sass" ? sassImportCode.join("\n") : sassImportCode.join(";\n");
        const useIndex = Math.max(
          src.lastIndexOf("@use "),
          src.lastIndexOf("@forward ")
        );
        const content = {
          code: src + "\n" + prefix,
          map: null
        };
        if (useIndex === -1) {
          content.code = prefix + src;
        } else {
          const newLineIndex = src.indexOf("\n", useIndex);
          if (newLineIndex !== -1) {
            const index = newLineIndex + 1;
            content.code = src.substring(0, index) + prefix + src.substring(index);
          }
        }
        return content;
      }
      return null;
    }
  };
}

const __QUASAR_VERSION__ = `'2.10.2'`;
const module = defineNuxtModule({
  meta: {
    name: "nuxt-quasar-vite",
    configKey: "quasar",
    compatibility: {
      nuxt: "^3.0.0-rc.2"
    }
  },
  defaults: {
    sassVariables: true,
    css: ["quasar/src/css/index.sass"],
    plugins: ["Notify"],
    config: {
      dark: true
    }
  },
  hooks: {
    "imports:sources": (presets) => {
      presets.push({
        from: "quasar/src/composables",
        imports: [
          "useQuasar",
          "useDialogPluginComponent",
          "useFormChild"
        ]
      }, {
        from: "quasar/src/utils",
        imports: [
          ["clone", "qclone"],
          ["colors", "qcolors"],
          ["copyToClipboard", "qcopyToClipboard"],
          ["createMetaMixin", "qcreateMetaMixin"],
          ["createUploaderComponent", "qcreateUploaderComponent"],
          ["date", "qdate"],
          ["debounce", "qdebounce"],
          ["dom", "qdom"],
          ["event", "qevent"],
          ["exportFile", "qexportFile"],
          ["extend", "qextend"],
          ["format", "qformat"],
          ["frameDebounce", "qframeDebounce"],
          ["getCssVar", "qgetCssVar"],
          ["noop", "qnoop"],
          ["morph", "qmorph"],
          ["openURL", "qopenURL"],
          ["patterns", "qpatterns"],
          ["scroll", "qscroll"],
          ["setCssVar", "qsetCssVar"],
          ["throttle", "qthrottle"],
          ["uid", "quid"]
        ]
      });
    },
    "components:dirs": async (dirs) => {
      const source = (await resolvePath("quasar/src/components")).replace(".js", "");
      dirs.push({
        path: source,
        transpile: true,
        watch: false,
        pattern: "*/Q*.js",
        pathPrefix: false
      });
    },
    "prepare:types": ({ references }) => {
      references.unshift({ types: "quasar" });
    }
  },
  setup: (opts, nuxt) => {
    nuxt.options.css.push(...opts.css);
    if (!nuxt.options.build.transpile.includes("quasar")) {
      nuxt.options.build.transpile.unshift("quasar");
    }
    nuxt.hook("vite:extendConfig", async (config, { isServer, isClient }) => {
      config.define = config.define || {};
      config.plugins = config.plugins || [];
      const define = {
        __QUASAR_VERSION__: `${__QUASAR_VERSION__}`,
        __QUASAR_SSR__: isServer,
        __QUASAR_SSR_SERVER__: isServer,
        __QUASAR_SSR_CLIENT__: isServer && isClient,
        __QUASAR_SSR_PWA__: false
      };
      Object.assign(config.define, define);
      if (opts.sassVariables) {
        const scssPath = typeof opts.sassVariables === "string" ? await resolvePath(opts.sassVariables) : true;
        const quasarScssTransform = getScssTransformPlugin(scssPath);
        config.plugins.push(quasarScssTransform);
      }
    });
    addPluginTemplate({
      filename: "plugins/quasarVueUse.ts",
      mode: "all",
      write: true,
      getContents: () => {
        const config = JSON.stringify(opts.config, null, 2);
        const plugins = opts.plugins.join(",");
        return `import installQ from 'quasar/src/install-quasar'
import { ${plugins} } from 'quasar/src/plugins'
import lang from 'quasar/src/lang'
import iconSet from 'quasar/src/icon-set'
import * as directives from 'quasar/src/directives'

export default defineNuxtPlugin((nuxtApp) => {
  const includes = {
    directives,
    plugins: { ${plugins} },
    config: ${config},
  }
  
  nuxtApp.vueApp.use({
    version: ${__QUASAR_VERSION__},
    install(app, opts) {
      if(process.server) {
        installQ(app, {...opts, ...includes}, nuxtApp.ssrContext.event)
      } else {
        installQ(app, {...opts, ...includes})
      }
    },
    lang,
    iconSet
  })
})
`;
      }
    });
  }
});

export { module as default };
